@@ -9,7 +9,7 @@
 */
 
 /* ====================================================================
- * Copyright (c) 1998-2001 Ralf S. Engelschall. All rights reserved.
+ * Copyright (c) 1998-2004 Ralf S. Engelschall. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -233,11 +233,17 @@ void ssl_init_Module(server_rec *s, pool *p)
 #ifdef SHARED_MODULE
     ssl_log(s, SSL_LOG_INFO, "Init: %snitializing %s library",
             mc->nInitCount == 1 ? "I" : "Rei", SSL_LIBRARY_NAME);
+#ifdef SSL_EXPERIMENTAL_ENGINE
+    ssl_init_Engine(s, p);
+#endif
     ssl_init_SSLLibrary();
 #else
     if (mc->nInitCount <= 2) {
         ssl_log(s, SSL_LOG_INFO, "Init: %snitializing %s library",
                 mc->nInitCount == 1 ? "I" : "Rei", SSL_LIBRARY_NAME);
+#ifdef SSL_EXPERIMENTAL_ENGINE
+        ssl_init_Engine(s, p);
+#endif
         ssl_init_SSLLibrary();
     }
 #endif
@@ -249,13 +255,6 @@ void ssl_init_Module(server_rec *s, pool *p)
 #endif
     }
 
-    /*
-     * SSL external crypto device ("engine") support
-     */
-#ifdef SSL_EXPERIMENTAL_ENGINE
-    ssl_init_Engine(s, p);
-#endif
-
     /*
      * Warn the user that he should use the session cache.
      * But we can operate without it, of course.
@@ -349,14 +348,14 @@ void ssl_init_Engine(server_rec *s, pool *p)
 
     if (mc->szCryptoDevice != NULL) {
         if ((e = ENGINE_by_id(mc->szCryptoDevice)) == NULL) {
-            ssl_log(s, SSL_LOG_ERROR, "Init: Failed to load Crypto Device API `%s'",
+            ssl_log(s, SSL_LOG_ERROR|SSL_ADD_SSLERR, "Init: Failed to load Crypto Device API `%s'",
                     mc->szCryptoDevice);
             ssl_die();
         }
         if (strEQ(mc->szCryptoDevice, "chil")) 
             ENGINE_ctrl(e, ENGINE_CTRL_CHIL_SET_FORKCHECK, 1, 0, 0);
         if (!ENGINE_set_default(e, ENGINE_METHOD_ALL)) {
-            ssl_log(s, SSL_LOG_ERROR, "Init: Failed to enable Crypto Device API `%s'",
+            ssl_log(s, SSL_LOG_ERROR|SSL_ADD_SSLERR, "Init: Failed to enable Crypto Device API `%s'",
                     mc->szCryptoDevice);
             ssl_die();
         }
@@ -419,7 +418,7 @@ void ssl_init_TmpKeysHandle(int action, server_rec *s, pool *p)
         asn1->nData  = i2d_DHparams(dh, NULL);
         asn1->cpData = ap_palloc(mc->pPool, asn1->nData);
         ucp = asn1->cpData; i2d_DHparams(dh, &ucp); /* 2nd arg increments */
-        /* no need to free dh, it's static */
+        DH_free(dh);
 
         /* import 1024 bit DH param */
         if ((dh = ssl_dh_GetTmpParam(1024)) == NULL) {
@@ -430,7 +429,7 @@ void ssl_init_TmpKeysHandle(int action, server_rec *s, pool *p)
         asn1->nData  = i2d_DHparams(dh, NULL);
         asn1->cpData = ap_palloc(mc->pPool, asn1->nData);
         ucp = asn1->cpData; i2d_DHparams(dh, &ucp); /* 2nd arg increments */
-        /* no need to free dh, it's static */
+        DH_free(dh);
     }
 
     /* Allocate Keys and Params */
@@ -603,6 +602,14 @@ void ssl_init_ConfigureServer(server_rec *s, pool *p, SSLSrvConfigRec *sc)
     else
         SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_SERVER);
 
+    /*
+     * Disallow a session from being resumed during a renegotiation,
+     * so that an acceptable cipher suite can be negotiated.
+     */
+#ifdef SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION
+    SSL_CTX_set_options(ctx, SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION);
+#endif
+
     /*
      *  Configure callbacks for SSL context
      */
@@ -796,6 +803,12 @@ void ssl_init_ConfigureServer(server_rec *s, pool *p, SSLSrvConfigRec *sc)
                     cpVHostID);
             ssl_die();
         }
+        if (!RSA_blinding_on(sc->pPrivateKey[SSL_AIDX_RSA]->pkey.rsa, NULL)) {
+            ssl_log(s, SSL_LOG_ERROR|SSL_ADD_SSLERR,
+                    "Init: (%s) Unable to enable RSA blinding (probably PRNG failure)",
+                    cpVHostID);
+            ssl_die();
+        }
         if (SSL_CTX_use_PrivateKey(ctx, sc->pPrivateKey[SSL_AIDX_RSA]) <= 0) {
             ssl_log(s, SSL_LOG_ERROR|SSL_ADD_SSLERR,
                     "Init: (%s) Unable to configure RSA server private key",
@@ -912,7 +925,7 @@ void ssl_init_CheckServers(server_rec *sm, pool *p)
     }
 
     /*
-     * Give out warnings when more than one SSL-aware virtual server uses the
+     * Give out warnings if more than one SSL-aware virtual server uses the
      * same IP:port. This doesn't work because mod_ssl then will always use
      * just the certificate/keys of one virtual host (which one cannot be said
      * easily - but that doesn't matter here).
@@ -924,6 +937,8 @@ void ssl_init_CheckServers(server_rec *sm, pool *p)
         sc = mySrvConfig(s);
         if (!sc->bEnabled)
             continue;
+        if (s->addrs == NULL)
+            continue;
         key = ap_psprintf(sp, "%pA:%u", &s->addrs->host_addr, s->addrs->host_port);
         ps = ssl_ds_table_get(t, key);
         if (ps != NULL) {
@@ -964,6 +979,7 @@ STACK_OF(X509_NAME) *ssl_init_FindCAList(server_rec *s, pool *pp, char *cpCAfile
     char *cp;
     pool *p;
     int n;
+    char buf[256];
 
     /*
      * Use a subpool so we don't bloat up the server pool which
@@ -983,13 +999,17 @@ STACK_OF(X509_NAME) *ssl_init_FindCAList(server_rec *s, pool *pp, char *cpCAfile
      */
     if (cpCAfile != NULL) {
         sk = SSL_load_client_CA_file(cpCAfile);
-        for(n = 0; sk != NULL && n < sk_X509_NAME_num(sk); n++) {
+        for (n = 0; sk != NULL && n < sk_X509_NAME_num(sk); n++) {
+            X509_NAME *name = sk_X509_NAME_value(sk, n);
             ssl_log(s, SSL_LOG_TRACE,
                     "CA certificate: %s",
-                    X509_NAME_oneline(sk_X509_NAME_value(sk, n), NULL, 0));
-            if (sk_X509_NAME_find(skCAList, sk_X509_NAME_value(sk, n)) < 0)
-                sk_X509_NAME_push(skCAList, sk_X509_NAME_value(sk, n));
+                    X509_NAME_oneline(name, buf, sizeof(buf)));
+            if (sk_X509_NAME_find(skCAList, name) < 0)
+                sk_X509_NAME_push(skCAList, name); /* will be freed when skCAList is */
+            else
+                X509_NAME_free(name);
         }
+        sk_X509_NAME_free(sk);
     }
 
     /*
@@ -1000,13 +1020,17 @@ STACK_OF(X509_NAME) *ssl_init_FindCAList(server_rec *s, pool *pp, char *cpCAfile
         while ((direntry = readdir(dir)) != NULL) {
             cp = ap_pstrcat(p, cpCApath, "/", direntry->d_name, NULL);
             sk = SSL_load_client_CA_file(cp);
-            for(n = 0; sk != NULL && n < sk_X509_NAME_num(sk); n++) {
+            for (n = 0; sk != NULL && n < sk_X509_NAME_num(sk); n++) {
+                X509_NAME *name = sk_X509_NAME_value(sk, n);
                 ssl_log(s, SSL_LOG_TRACE,
                         "CA certificate: %s",
-                        X509_NAME_oneline(sk_X509_NAME_value(sk, n), NULL, 0));
-                if (sk_X509_NAME_find(skCAList, sk_X509_NAME_value(sk, n)) < 0)
-                    sk_X509_NAME_push(skCAList, sk_X509_NAME_value(sk, n));
+                        X509_NAME_oneline(name, buf, sizeof(buf)));
+                if (sk_X509_NAME_find(skCAList, name) < 0)
+                    sk_X509_NAME_push(skCAList, name);
+                else
+                    X509_NAME_free(name);
             }
+            sk_X509_NAME_free(sk);
         }
         ap_pclosedir(p, dir);
     }
@@ -1055,6 +1079,10 @@ void ssl_init_ModuleKill(void *data)
      */
     for (; s != NULL; s = s->next) {
         sc = mySrvConfig(s);
+        if (sc->pRevocationStore != NULL) {
+            X509_STORE_free(sc->pRevocationStore);
+            sc->pRevocationStore = NULL;
+        }
         if (sc->pPublicCert[SSL_AIDX_RSA] != NULL) {
             X509_free(sc->pPublicCert[SSL_AIDX_RSA]);
             sc->pPublicCert[SSL_AIDX_RSA] = NULL;
